<?php
/**
 * Admin class
 *
 * @author  YITH <plugins@yithemes.com>
 * @package YITH\Deposits\Classes
 * @version 1.0.0
 */

if ( ! defined( 'YITH_WCDP' ) ) {
	exit;
} // Exit if accessed directly

if ( ! class_exists( 'YITH_WCDP_Admin' ) ) {
	/**
	 * WooCommerce Deposits / Down Payments Admin
	 *
	 * @since 1.0.0
	 */
	class YITH_WCDP_Admin {

		/**
		 * Single instance of the class
		 *
		 * @var \YITH_WCDP_Admin
		 * @since 1.0.0
		 */
		protected static $instance;

		/**
		 * The panel
		 *
		 * @var YIT_Plugin_Panel_WooCommerce $panel
		 */
		protected $panel;

		/**
		 * An array of meta available for the product
		 *
		 * @var array
		 */
		protected $product_meta = array();

		/**
		 * Docs url
		 *
		 * @var string Official documentation url
		 * @since 1.0.0
		 */
		public $doc_url = 'https://yithemes.com/docs-plugins/yith-woocommerce-deposits-and-down-payments/';

		/**
		 * Premium landing url
		 *
		 * @var string Premium landing url
		 * @since 1.0.0
		 */
		public $premium_landing_url = 'https://yithemes.com/themes/plugins/yith-woocommerce-deposits-and-down-payments/';

		/**
		 * List of available tab for deposit panel
		 *
		 * @var array
		 * @access public
		 * @since  1.0.0
		 */
		public $available_tabs = array();

		/**
		 * Constructor method
		 *
		 * @since 1.0.0
		 */
		public function __construct() {
			add_action( 'admin_init', array( $this, 'init' ) );

			// register plugin panel.
			add_action( 'admin_menu', array( $this, 'register_panel' ), 5 );
			add_action( 'admin_enqueue_scripts', array( $this, 'enqueue' ) );

			// register plugin links & meta row.
			add_filter( 'plugin_action_links_' . YITH_WCDP_INIT, array( $this, 'action_links' ) );
			add_filter( 'yith_show_plugin_row_meta', array( $this, 'add_plugin_meta' ), 10, 5 );
		}

		/**
		 * Init admin functionalities
		 */
		public function init() {
			// init functions.
			YITH_WCDP_Ajax_Handler::init();
			YITH_WCDP_Admin_Products::init();
			YITH_WCDP_Admin_Notices::init();

			// create instances.
			new YITH_WCDP_Admin_Orders();
			new YITH_WCDP_Admin_Panel();
		}

		/* === PANEL METHODS === */

		/**
		 * Return array of screen ids for affiliate plugin
		 *
		 * @param string $context Context of the operation.
		 *
		 * @return mixed Array of available screens
		 * @since 1.0.0
		 */
		public function get_screen_ids( $context = 'view' ) {
			$screen_ids = array(
				'yith-plugins_page_yith_wcdp_panel',
			);

			// in case we're checking for enqueue operation, append additional screens.
			if ( 'enqueue' === $context ) {
				$screen_ids = array_merge(
					$screen_ids,
					array(
						'edit-shop_order',
						'shop_order',
						'product',
						'woocommerce_page_wc-orders',
					)
				);
			}

			/**
			 * APPLY_FILTERS: yith_wcdp_screen_ids
			 *
			 * Filters available screen IDs for Affiliate's plugin.
			 *
			 * @param mixed $screen_ids Array of available screens.
			 *
			 * @return mixed
			 */
			return apply_filters( 'yith_wcdp_screen_ids', $screen_ids );
		}

		/**
		 * Returns true if we're on plugin screen
		 *
		 * @param string $context Context of the operation.
		 *
		 * @return bool Whether we're on plugin's own screen or not
		 */
		public function is_own_screen( $context = 'view' ) {
			$screen = get_current_screen();

			if ( ! $screen ) {
				return false;
			}

			return apply_filters( 'yith_wcdp_is_own_screen', in_array( $screen->id, $this->get_screen_ids( $context ), true ) );
		}

		/**
		 * Returns an array of available tabs for current plugin
		 *
		 * @return array Array of available tabs
		 */
		public function get_available_tabs() {
			if ( ! $this->available_tabs ) {
				// sets available tab.
				/**
				 * APPLY_FILTERS: yith_wcdp_available_admin_tabs
				 *
				 * Filters admin available tabs.
				 *
				 * @param array Array of tabs.
				 *
				 * @return array
				 */
				$this->available_tabs = apply_filters(
					'yith_wcdp_available_admin_tabs',
					array(
						'settings' => __( 'General Options', 'yith-woocommerce-deposits-and-down-payments' ),
						'deposits' => __( 'Deposit Rules', 'yith-woocommerce-deposits-and-down-payments' ),
					)
				);
			}

			return $this->available_tabs;
		}

		/**
		 * Register panel
		 *
		 * @return void
		 * @since 1.0.0
		 */
		public function register_panel() {
			if ( ! empty( $this->panel ) ) {
				return;
			}

			$help_tab = array(
				'hc_url' => 'https://support.yithemes.com/hc/en-us/categories/360003474878-YITH-WOOCOMMERCE-DEPOSITS-AND-DOWN-PAYMENTS',
			);

			$welcome_modals = array(
				'show_in'  => 'panel',
				'on_close' => function () {
					update_option( 'yith_wcdp_welcome_modal_status', 'no' );
				},
				'modals'   => array(
					'welcome' => array(
						'type'        => 'welcome',
						'description' => __( 'Now you can allow your customers to pay a deposit to book products or services and pay the balance at a later time.', 'yith-test-plugin' ),
						'show'        => get_option( 'yith_wcdp_welcome_modal_status', 'welcome' ) === 'welcome',
						'items'       => array(
							'documentation'      => array(),
							'stripe-integration' => array(
								'url'         => 'https://yithemes.com/themes/plugins/yith-woocommerce-stripe/',
								'title'       => __( 'Optional: <mark>get our Stripe plugin</mark> to unlock advanced features', 'yith-woocommerce-deposits-and-down-payments' ),
								'description' => __( 'Use Stripe to automatically charge the orders\' balance to your customers\' credit cards', 'yith-woocommerce-deposits-and-down-payments' ),
							),
							'settings-panel'     => array(
								'url'         => YITH_WCDP_Admin()->get_panel_url(),
								'title'       => __( 'Are you ready? Start by <mark>configuring your deposit options</mark>', 'yith-woocommerce-deposits-and-down-payments' ),
								'description' => __( '... and start the adventure!', 'yith-woocommerce-deposits-and-down-payments' ),
							),
						),
					),
					'update'  => array(
						'type'  => 'update',
						'show'  => get_option( 'yith_wcdp_welcome_modal_status', 'welcome' ) === 'update',
						'since' => '2.0',
						'items' => array(
							'deposit-rules'       => array(
								'url'         => YITH_WCDP_Admin()->get_panel_url( 'deposits' ),
								'title'       => __( 'Welcome our "Deposit Rules" concept', 'yith-woocommerce-deposits-and-down-payments' ),
								'description' => __( 'Override the global amount by creating advanced rules and setting different values for specific user roles, products, or categories.', 'yith-woocommerce-deposits-and-down-payments' ),
							),
							'booking-integration' => array(
								'url'         => 'https://yithemes.com/themes/plugins/yith-woocommerce-booking/',
								'title'       => __( 'New integration: YITH Booking and Appointment for WooCommerce', 'yith-woocommerce-deposits-and-down-payments' ),
								'description' => __( 'Allow your customers to pay a deposit to book products or services, and ask them to pay for the balance on the booking start date or XX days before that date.', 'yith-woocommerce-deposits-and-down-payments' ),
								'cta'         => __( 'Discover our YITH Booking and Appointment for WooCommerce plugin &gt;', 'yith-woocommerce-deposits-and-down-payments' ),
							),
							'stripe-integration'  => array(
								'url'         => 'https://yithemes.com/themes/plugins/yith-woocommerce-stripe/',
								'title'       => __( 'New integration: YITH WooCommerce Stripe', 'yith-woocommerce-deposits-and-down-payments' ),
								'description' => __( 'Automatically charge the balance amount to the same credit card used by the customer to pay for the deposit. This feature allows you to lower the number of unpaid orders and simplify the purchase process.', 'yith-woocommerce-deposits-and-down-payments' ),
								'cta'         => __( 'Discover our YITH WooCommerce Stripe plugin &gt;', 'yith-woocommerce-deposits-and-down-payments' ),
							),
						),
					),
				),

			);

			$args = apply_filters(
				'yith_wcdp_admin_menu_args',
				array(
					'admin-tabs'       => $this->get_available_tabs(),
					'capability'       => 'manage_options',
					'class'            => yith_set_wrapper_class( 'yith-plugin-ui--classic-wp-list-style' ),
					'create_menu_page' => true,
					'help_tab'         => $help_tab,
					'is_extended'      => false,
					'is_premium'       => true,
					'menu_title'       => 'Deposits / Down Payments',
					'options-path'     => YITH_WCDP_DIR . 'plugin-options',
					'page'             => 'yith_wcdp_panel',
					'page_title'       => 'YITH WooCommerce Deposits / Down Payments',
					'parent'           => '',
					'parent_page'      => 'yith_plugin_panel',
					'parent_slug'      => '',
					'plugin_icon'      => YITH_WCDP_URL . 'assets/images/icon.svg',
					'plugin_slug'      => YITH_WCDP_SLUG,
					'plugin_version'   => YITH_WCDP::VERSION,
					'welcome_modals'   => $welcome_modals,
				)
			);

			if ( ! class_exists( 'YIT_Plugin_Panel_WooCommerce' ) ) {
				require_once YITH_WCDP_DIR . 'plugin-fw/lib/yit-plugin-panel-wc.php';
			}

			$this->panel = new YIT_Plugin_Panel_WooCommerce( $args );
		}

		/**
		 * Returns url to panel page
		 *
		 * @param string $tab    Tab of the panel to link.
		 * @param array  $params Params to add to url.
		 *
		 * @return string Formatted panel url.
		 */
		public function get_panel_url( $tab = '', $params = array() ) {
			$url = add_query_arg( 'page', 'yith_wcdp_panel', admin_url( 'admin.php' ) );

			if ( ! empty( $tab ) ) {
				$url = add_query_arg( 'tab', $tab, $url );
			}

			if ( ! empty( $params ) ) {
				$url = add_query_arg( $params, $url );
			}

			return $url;
		}

		/**
		 * Enqueue admin side scripts
		 *
		 * @return void
		 * @since 1.0.0
		 */
		public function enqueue() {
			if ( ! $this->is_own_screen( 'enqueue' ) ) {
				return;
			}

			$suffix = ( defined( 'WP_DEBUG' ) && WP_DEBUG ) ? '' : '.min';

			// register scripts.
			wp_register_script( 'yith-wcdp', YITH_WCDP_URL . 'assets/js/admin/yith-wcdp.bundle' . $suffix . '.js', array( 'jquery', 'jquery-ui-datepicker', 'wp-mediaelement', 'yith-plugin-fw-fields' ), YITH_WCDP::YITH_WCDP_VERSION, true );
			wp_localize_script(
				'yith-wcdp',
				'yith_wcdp',
				array(
					'labels' => array(
						'add_rule_title'  => __( 'Add rule', 'yith-woocommerce-deposits-and-down-payments' ),
						'edit_rule_title' => __( 'Edit rule', 'yith-woocommerce-deposits-and-down-payments' ),
					),
				)
			);

			// enqueue scripts.
			/**
			 * DO_ACTION: yith_wcdp_before_admin_script_enqueue
			 *
			 * Action triggered before enqueueing 'yith-wcdp' admin script.
			 */
			do_action( 'yith_wcdp_before_admin_script_enqueue' );
			wp_enqueue_script( 'yith-wcdp' );

			// register styles.
			wp_register_style( 'yith-wcdp', YITH_WCDP_URL . 'assets/css/yith-wcdp-admin.css', array( 'yith-plugin-fw-fields' ), YITH_WCDP::YITH_WCDP_VERSION );

			// enqueue styles.
			/**
			 * DO_ACTION: yith_wcdp_before_admin_style_enqueue
			 *
			 * Action triggered before enqueueing 'yith-wcdp' admin style.
			 */
			do_action( 'yith_wcdp_before_admin_style_enqueue' );
			wp_enqueue_style( 'yith-wcdp' );
		}

		/* === PLUGIN LINK METHODS === */

		/**
		 * Get the premium landing uri
		 *
		 * @return  string The premium landing link
		 * @since   1.0.0
		 */
		public function get_premium_landing_uri() {
			return $this->premium_landing_url;
		}

		/**
		 * Add plugin action links
		 *
		 * @param array $links Plugins links array.
		 *
		 * @return array Filtered link array
		 * @since 1.0.0
		 */
		public function action_links( $links ) {
			$links = yith_add_action_links( $links, 'yith_wcdp_panel', defined( 'YITH_WCDP_PREMIUM_INIT' ), YITH_WCDP_SLUG );

			return $links;
		}

		/**
		 * Adds plugin row meta
		 *
		 * @param array    $new_row_meta_args  New arguments.
		 * @param string[] $plugin_meta        An array of the plugin's metadata, including the version, author, author URI, and plugin URI.
		 * @param string   $plugin_file        Path to the plugin file relative to the plugins directory.
		 * @param array    $plugin_data        An array of plugin data.
		 * @param string   $status             Status filter currently applied to the plugin list.
		 * @param string   $init_file          Constant with plugin_file.
		 *
		 * @return array Filtered array of plugin meta
		 * @since 1.0.0
		 */
		public function add_plugin_meta( $new_row_meta_args, $plugin_meta, $plugin_file, $plugin_data, $status, $init_file = 'YITH_WCDP_INIT' ) {
			if ( defined( $init_file ) && constant( $init_file ) === $plugin_file ) {
				$new_row_meta_args['slug'] = YITH_WCDP_SLUG;

				if ( defined( 'YITH_WCDP_PREMIUM_INIT' ) ) {
					$new_row_meta_args['is_premium'] = true;
				}
			}

			return $new_row_meta_args;
		}

		/**
		 * Returns single instance of the class
		 *
		 * @return \YITH_WCDP_Admin
		 * @since 1.0.0
		 */
		public static function get_instance() {
			if ( is_null( self::$instance ) ) {
				self::$instance = new self();
			}

			return self::$instance;
		}
	}
}

/**
 * Unique access to instance of YITH_WCDP_Admin class
 *
 * @return \YITH_WCDP_Admin
 * @since 1.0.0
 */
function YITH_WCDP_Admin() { // phpcs:ignore WordPress.NamingConventions.ValidFunctionName.FunctionNameInvalid, Universal.Files.SeparateFunctionsFromOO
	return YITH_WCDP_Admin::get_instance();
}

/**
 * Legacy function, left just for backward compatibility
 *
 * @return \YITH_WCDP_Admin
 */
function YITH_WCDP_Admin_Premium() { // phpcs:ignore WordPress.NamingConventions.ValidFunctionName.FunctionNameInvalid
	_deprecated_function( 'YITH_WCDP_Admin_Premium', '2.0.0', 'YITH_WCDP_Admin' );

	return YITH_WCDP_Admin();
}

// create legacy class alias.
class_alias( 'YITH_WCDP_Admin', 'YITH_WCDP_Admin_Premium' );
